#include <windows.h>
#pragma comment(lib, "kernel32.lib")
#include <mex.h>
#include "ExampleClass.hpp"

#ifndef max
//! not defined in the C standard used by visual studio
#define max(a,b) (((a) > (b)) ? (a) : (b))
#endif
#ifndef min
//! not defined in the C standard used by visual studio
#define min(a,b) (((a) < (b)) ? (a) : (b))
#endif

//*******************************************************************************************
void mexFunction(int nlhs, mxArray *plhs[], int	nrhs, const	mxArray	*prhs[]) {
	/*!
	 *  \brief Entry point in the code for Matlab.  Equivalent to main().
	 *  \param nlhs number of left hand mxArrays to return
	 *  \param plhs array of pointers to the output mxArrays
	 *  \param nrhs number of input mxArrays
	 *  \param prhs array of pointers to the input mxArrays.
	 */
	//declare all vars
	float *array_a = 0, *array_b = 0;
	const mwSize *array_a_size = 0, *array_b_size = 0;
	size_t Ndim_a = 0, Ndim_b = 0;

	//check for required inputs, correct types, and dimensions
	if (nrhs != 2)
		mexErrMsgIdAndTxt("CUDAtemplate:WrongNumberInputs", "Input must include vectors a,b");

	if (mxGetClassID(prhs[0]) != mxSINGLE_CLASS)
		mexErrMsgIdAndTxt("CUDAtemplate:WrongType", "Inputs must be 32 bit ints");

	Ndim_a = mxGetNumberOfDimensions(prhs[0]);
	Ndim_b = mxGetNumberOfDimensions(prhs[1]);
	array_a_size = mxGetDimensions(prhs[0]);
	array_b_size = mxGetDimensions(prhs[1]);

	//1D vectors still return 2D
	if (Ndim_a > 2 || Ndim_b > 2 || array_a_size[0] > 1 || array_b_size[0] > 1)
		mexErrMsgIdAndTxt("CUDAtemplate:WrongDimensions", "a and b should be 1D vectors");

	if (array_a_size[1] != array_b_size[1])
		mexErrMsgIdAndTxt("CUDAtemplate:VectorMismatch", "size of a != b");
	//retrieve all inputs
	array_a = (float*)mxGetData(prhs[0]);
	array_b = (float*)mxGetData(prhs[1]);

	//allocate results memory and copy results back to matlab
	const size_t bytes = array_a_size[1] * sizeof(float);
	plhs[0] = mxCreateNumericMatrix(array_a_size[0], array_a_size[1], mxSINGLE_CLASS, mxREAL);

	//clear any cuda errors and reset device
	cudaDeviceReset();

	//work with class containing cuda calls
	Example EC ((int) array_a_size[1], array_a, array_b);
	EC.add();
	cudaMemcpy((float*)mxGetData(plhs[0]), EC.d_C, bytes, cudaMemcpyDeviceToHost);
	EC.CudaError("Memcpy out");

	//clean up!
	cudaThreadExit(); //release context so future cudaSetDevice calls work

	return;
 }